/* Generated by Together */

package bemessung.betontraeger;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Vector;

/**
 * Datenbank der Betonsorten nach den Nenngren der DIN 1045-1 geordnet. Wobei ein unter einem Schlssel abgelegtes Feld folgendermaen aufgebaut ist:
 * f(ck) in [MN/m^2] , epsilon(c2) in [/] , epsilon (c2u) in [/] , Grundwert rho
 */
final class BetonTypen {

	protected void BetonTypen() {}

    protected final static HashMap betonTypen = new HashMap();

    static{
       betonTypen.put(new String("C12/15"),new double[] {12 ,  -0.002 , -0.0035 , 0.51} );
       betonTypen.put(new String("C16/20"),new double[] {16 ,  -0.002 , -0.0035 , 0.61} );
       betonTypen.put(new String("C20/25"),new double[] {20 ,  -0.002 , -0.0035 , 0.70} );
       betonTypen.put(new String("C25/30"),new double[] {25 ,  -0.002 , -0.0035 , 0.83} );
       betonTypen.put(new String("C30/37"),new double[] {30 ,  -0.002 , -0.0035 , 0.93} );
       betonTypen.put(new String("C35/45"),new double[] {35 ,  -0.002 , -0.0035 , 1.02} );
       betonTypen.put(new String("C40/50"),new double[] {40 ,  -0.002 , -0.0035 , 1.12} );
       betonTypen.put(new String("C45/55"),new double[] {45 ,  -0.002 , -0.0035 , 1.21} );
       betonTypen.put(new String("C50/60"),new double[] {50 ,  -0.002 , -0.0035 , 1.31} );
    }


    /**
     * berprft, ob es einen Bertontyp gibt oder nicht.
     * @param key Betonbezeichnung zum Beispiel C12/15
     * @return true wenn es den Typ gibt
     * @return false wenn es den Typ nicht gibt
     */
    static boolean isBetonTyp(String key){
        return betonTypen.containsKey(key);
    }

    /**
     * Liefert die Zylinderdruckfestigkeit f(ck) zurck, wenn es den Betontyp gibt.
     * @param key Betontyp zum Beispiel C12/15
     * @return Zylinderdruckfestigkeit f(ck)
     * @return -1 wenn der Betontyp nicht existiert
     */
    static double getFck(String key){
      	return (isBetonTyp(key) ? ((double[])(betonTypen.get(key)))[0] : -1.0);
    }

    /**
     * Liefert die Dehnung epsilon(c2) zurck, wenn es den Betontyp gibt.
     * @param key Betontyp zum Beispiel C12/15
     * @return Zylinderdruckfestigkeit epsilon(c2)
     * @return -1 wenn der Betontyp nicht existiert
     */
    static double getEpsilonC2(String key){
      	return (isBetonTyp(key) ? ((double[])(betonTypen.get(key)))[1] : -1.0);
    }

    /**
     * Liefert die Dehnung epsilon(c2u) zurck, wenn es den Betontyp gibt.
     * @param key Betontyp zum Beispiel C12/15
     * @return Zylinderdruckfestigkeit epsilon(c2u)
     * @return -1 wenn der Betontyp nicht existiert
     */
    static double getEpsilonC2u(String key){
      	return (isBetonTyp(key) ? ((double[])(betonTypen.get(key)))[2] : -1.0);
    }

    /**
     * Liefert den Grundwert Rho fr die
     * Ermittlung der Mindestbewehrung, wenn es den Betontyp gibt.
     * @param key Betontyp zum Beispiel C12/15
     * @return Roh
     * @return -1 wenn der Betontyp nicht existiert
     */
    static double getRoh(String key){
      	return (isBetonTyp(key) ? ((double[])(betonTypen.get(key)))[3] : -1.0);
    }


}
