/* Generated by Together */

package bemessung.betontraeger;

/**
 * Dieser Visitor fhrt die Schubbemessung nach DIN 1045-1 durch.
 * getestet und fr gut befunden johannes hhne
 */
final class Schubbewehrung implements BewehrungsVisitor {

   	//Vorgaben aus dem Element
    protected double b   = 0.0;
    protected double h   = 0.0;
    protected double d1  = 0.0;
    protected double as  = 0.0;
    protected double N   = 0.0;
    protected double Vsd = 0.0;
    protected double fck = 0.0;
    protected double fyd = 0.0;
    protected double gammaC = 0.0;
    protected double rho = 0.0; //Grundwert der Mindestbewehrung

    //zu errechnende Gren
    protected double Vrdct = 0.0;
    protected double Ac    = 0.0;
    protected double fcd   = 0.0;
    protected double az	   = 0.0;
    protected double Vrdmax = 0.0;
    protected double cotT  = 0.0; //neigung der Druckstrebe

    public void visitBalkenBean(BalkenBean element){
        b   = element.getB();
        h   = element.getH();
        d1  = element.getD1();
        as  = element.getAz();
        N   = element.getN();
        Vsd = element.getQ();
        fck = BetonTypen.getFck(element.getBeton());
        fyd = element.getFyk()/element.getGammaS();
        gammaC = element.getGammaC();
        rho = BetonTypen.getRoh(element.getBeton());

        element.setAq(ablaufkontrolle());
    }

    public void visitStuetzenBean(StuetzenBean element){
        b   = element.getB();
        h   = element.getH();
        d1  = element.getD1();
        as  = element.getAz();
        N   = element.getN();
        Vsd = element.getQ();
        fck = BetonTypen.getFck(element.getBeton());
        fyd = element.getFyk()/element.getGammaS();
        gammaC = element.getGammaC();
        rho = BetonTypen.getRoh(element.getBeton());

        element.setAq(ablaufkontrolle());
    }

	/**
     * Diese Method steuert den Ablauf der Berechnung
     */
    protected double ablaufkontrolle(){
        Ac = b*h;
		fcd = 0.85*fck/gammaC;

		qOhneRechBewehrung();

        if(Vrdct < Vsd){
			druckstreben();
            if(Vrdmax >= Vsd){
                return noetigeBewehrung();
            }
            else{
                return -1.0;
            }
        }
        else{
            return mindestBewehrung();
        }

    }

    protected double noetigeBewehrung(){
		return (Vsd/(cotT*fyd*h));
    }
    protected double mindestBewehrung(){
		return (b <= h ? (2*rho*b):(1.3*rho*b));
    }


    /**
     * Hier wird die maximale Querkraft ohne rechnerische
     * Bewehrung ermittelt
     */
    protected void qOhneRechBewehrung(){
        double k = 1+Math.sqrt(200/(h*1000)) <= 2 ? 1+Math.sqrt(200/(h*1000)) : 2;
		double rho1 = as/(Ac) <= 0.02 ? as/(Ac) : 0.02;
		Vrdct = (0.1*k*Math.pow((100*rho1*fck) , 0.3333333333) - 0.12*N/(Ac))*b*h;

    }

    /**
     * Hier werden die Druckstreben nachgewiesen
     */
    protected void druckstreben(){
		double Vrdc = (0.24*Math.pow(fck , 0.33333333)*(1+1.2*(N/(Ac*fcd))))*b*h;
		cotT = (1.2-1.4*N/(Ac*fcd))/(1-Vrdc/Vsd)>= 1.0 ? (1.2-1.4*N/(Ac*fcd))/(1-Vrdc/Vsd): 1.0;
        if(cotT >= 3.0)
            cotT = 3.0;
        Vrdmax = (0.75*fcd*b*h*cotT)/(cotT*cotT+1);
    }

}
