/* Generated by Together */

package bemessung.betontraeger;

import java.lang.Math;

/**
 * Dieser Visitor berechnet die ntige Zugbewehrung fr einen gegebenen Balkenquerschnitt.
 * Die Klasse wurde getestet und fr gut befunden
 * @author Johannes Hhne
 */
final class Zugbewehrung implements BewehrungsVisitor {

    	//Variablen
        protected BalkenBean element;
        protected double M   = 0.0; //Moment
        protected double N   = 0.0; //Normalkraft
		protected double d1  = 0.0;
        protected double z   = 0.0; //innerer Hebel
		protected double b   = 0.0; // Breite des Querschnitts
		protected double h   = 0.0; //statische Hhe des Querschnitts

   		protected final double alpha = 0.85;  //Alpha fr Normalbeton
		protected double fcd = 0.0; //Betonbruchspannung
		protected double epsilonC2 = 0.0;
		protected double epsilonC2u = 0.0;

		//Iterationsgren
        protected double a = 0.0;
		protected double kx = 0.0;
        protected double ka = 0.0;
        protected double epsilonD = 0.0;
        protected double sigmaZ = 0.0;
        protected double links = 0.0;
        protected double rechts = 0.0;
        protected double delta = 0.0;


    /**
     * Iteriert die Dehnungen des Balkens solange bis sich ein Gleichgewicht
     * einstellt, das der Vertglichkeitsbedingung gengt.
     * @param element BalkenBean
     */
    public void visitBalkenBean(BalkenBean element){
        this.element = element;
        M   = element.getM(); //Moment
        N   = element.getN(); //Normalkraft
		d1	 = element.getD1();
        z   = (element.getH()+d1)/2 - d1; //innerer Hebel
		b   = element.getB(); // Breite des Querschnitts
		h   = element.getH(); //statische Hhe des Querschnitts

        fcd = BetonTypen.getFck(element.getBeton()) * alpha / element.getGammaC(); //Betonbruchspannung
		epsilonC2 = BetonTypen.getEpsilonC2((element.getBeton()));
		epsilonC2u = BetonTypen.getEpsilonC2u(element.getBeton());

        //linke Seite
        links =(M - N*z) / (fcd*b*h*h)  ;

        //Schleife ber die Dehnungen fr die rechte Seite
        boolean trigger = false; //Schalter fr Iterationsabbruch
        loop:
        for(double epsilonCg = -0.0035 ; epsilonCg <= 0.0 ; epsilonCg+=0.00001){
			for(double epsilonSg = 0.0 ; epsilonSg <= 0.025 ; epsilonSg+=0.00001){

                kern(epsilonCg , epsilonSg);

                //Abbruchschranke
				if(delta < 0.0001 && delta > -0.0001){
                    trigger = true;
                    double az = 1/sigmaS(epsilonSg , element) * (N + (M-N*z)/(h*(1-ka*kx)));
					element.setAz(az);
                    break loop;
                }
        	}
        }
		// Wenn az nicht  emitteltwerden konnten wird die stahlflche auf -1.0 gesetzt
        if(trigger == false)
            element.setAz(-1.0);
    }
	/**
     * Die Methode wird nicht bentigt, da Sttzen symmetrisch bemessen werde.
     * @param element StuetzenBean
     * @throws MethodNotUsefullException
     */
    public void visitStuetzenBean(StuetzenBean element)throws Exception{
        throw(new Exception("MethodNotUsefull"));
    }

    /**
     * Berechnet alle Gren der Iteration
     * @param epsilon(cg)
     * @param epsilon(c2)n Fehler aufgetreten ist
     */
    protected void kern(double epsilonCg , double epsilonSg) {
    	//Variablen
        a  = alpha(epsilonCg , epsilonC2 , epsilonC2u);
        kx = kx(epsilonCg , epsilonSg);
        ka = ka(epsilonCg , epsilonC2 , epsilonC2u);
        sigmaZ = sigmaS(epsilonSg, element);





        //rechte Seite
		rechts = a*kx*(1-ka*kx) ;

        //Deltabereich
        delta = links - rechts;



    }

    /**
     * Berechnung von alpha in Abhngigkeit der Dehnungen.
     * @param epsilon(cg)
     * @param epsilon(c2)
     * @param epsilon(c2u)
     * @return alpha
     * @return -1 wenn ein Fehler aufgetreten ist
     */
    protected double alpha(double epsilonCg , double epsilonC2, double epsilonC2u) {
		if(Math.abs(epsilonCg) <= Math.abs(epsilonC2))
            return (epsilonCg/epsilonC2 - epsilonCg*epsilonCg/(3*epsilonC2*epsilonC2));
        if(Math.abs(epsilonC2) < Math.abs(epsilonCg) && Math.abs(epsilonCg) <= Math.abs(epsilonC2u))
            return (1 - epsilonC2/(3*epsilonCg));
		else
        	return -1;
    }

    /**
     * Berechnung von k(a) in Abhngigkeit der Dehnungen.
     * @param epsilon(cg)
     * @param epsilon(c2)
     * @param epsilon(c2u)
     * @return k(a)
     * @return -1 wenn ein Fehler aufgetreten ist
     */
    protected double ka(double epsilonCg , double epsilonC2, double epsilonC2u) {
        if(Math.abs(epsilonCg) <= Math.abs(epsilonC2))
            return ((1 - epsilonCg/(4*epsilonC2)) / (3 - epsilonCg/epsilonC2));
        if(Math.abs(epsilonC2) < Math.abs(epsilonCg) && Math.abs(epsilonCg) <= Math.abs(epsilonC2u))
            return (1 - (3*epsilonCg/(3*epsilonCg - epsilonC2) *
                       (6*epsilonCg*epsilonCg - epsilonC2*epsilonC2)/(12*epsilonCg*epsilonCg)));
		else
        	return -1;
    }

    /**
     * Berechnung von k(x) in Abhngigkeit der Dehnungen.
     * @param epsilon(cg)
     * @param epsilon(sg)
     * @return k(x)
     * @return -1 wenn ein Fehler aufgetreten ist
     */
    protected double kx(double epsilonCg , double epsilonSg) {
        return (epsilonCg < 0 &&  epsilonCg != epsilonSg ? 1 / (1 - epsilonSg/epsilonCg) : -1 );
    }

    /**
     * Berechnung der Stahlspannung in Abhngigkeit der Dehnung
     * @param epsilon(sg) in [/]
     * @param element vom Typ BalkenBean
     * @return sigma(s) in [MN/m^2]
     */
    protected double sigmaS( double epsilonSg , BalkenBean element){
        return(epsilonSg < (element.getFyk()/element.getGammaS()/200000) ? 200000*epsilonSg : element.getFyk()/element.getGammaS());
    }
}
