/* Generated by Together */

package bemessung.eckverbindungen;

/**
 * Diese Visitor Klasse Bemisst eine Biegesteife geschraubte Eckverbindung
 * @author Johannes Hhne
 * Geprft und fr gut befunden
 */
public class Bemessung implements Visitor {
    public void visitBiegesteifBean(BiegesteifBean eck){
		/**
		 * Normalkraft im Riegel in [MN]
		 */
		final double n_r = eck.getN_r();
		/**
		 * Moment im Riegel in [MNm]
		 */
		final double m_r = eck.getM_r();
		/**
		 * Querkraft im Riegel in [MN]
		 */
		final double q_r = eck.getQ_r();
		/**
		 * Neigung des Riegels
		 */
		final double alp = eck.getAlpha();
		/**
		 * die I-Profilhhe des Riegels in [m]
		 */
		final double h_r = eck.getH_r()/1000;
		/**
		 * die I-Profilhhe des Stiels  in [m]
		 */
		final double h_s = eck.getH_s()/1000;
		/**
		 * Die Flanschdicke des Riegels in [m]
		 */
		final double t_r = eck.getT_r()/1000;
		/**
		 * die Flanschbreite der Riegels in [m]
		 */
		final double b_r = eck.getB_r()/1000;
		/**
		 * Anzahl der Schrauben pro Achse
		 */
		final int n = eck.getN();
		/**
		 * der Lochdurchmesser in [m]
		 */
		final double d = eck.getD()/1000;
		/**
		 * Schraubenbezeichnung M12 , M16 ... M36
		 */
		final String d_schr = eck.getD_schr();
		/**
		 * Beilegscheibendurchmesser in [m]
		 */
		final double d_sch = eck.getD_sch()/1000;
		/**
		 * die Dicke der Stirnplatte in [m]
		 */
		final double d_pl = eck.getD_pl()/1000;
		/**
		 * der obere Schraubenabstand in [m]
		 */
		final double e4 = eck.getE4()/1000;
		/**
		 * Der berstand der Strirnplatte in [m]
		 */
		final double ue = eck.getUe()/1000;
		/**
		 * Teilsicherheitsbeiwert gamma(M)
		 */
		final double gammaM = eck.getGammaM();
		/**
		 * Streckgrenze f(y,k) in [MN/m^2]
		 */
		final double fyk = eck.getFyk();
	
		//Zu errechnende Gren
		//alaha im Bogenma
		final double alpha = Math.atan(alp);
//        System.out.println("alpha = "+alpha);


		//Rahmenkrfte im Riegel
		final double Nr = q_r*Math.sin(alpha) + n_r*Math.cos(alpha);
		final double Vr = q_r*Math.cos(alpha) - n_r*Math.sin(alpha);
		final double Mr = m_r+Vr*h_s/2.0;
//        System.out.println("M(r) ="+Mr+" N(r) = "+Nr+" V(r) = "+Vr+" hs/2 = "+h_s/2);
	
		//Aufteilung der Krfte auf die Flansche des Riegels
		final double N_RM = Mr/(h_r-t_r);
		final double Zrd = Nr/2-N_RM;
//		final double Drd = -Nr/2-N_RM;
//        System.out.println("Z(d) = "+Zrd);
	
		//Vorwerte fr die Bemessung
		final double M1pld = 1.1*fyk/gammaM*b_r*t_r*t_r/4;
		final double Zpld  = b_r*t_r*fyk/gammaM;
		final double M2pld = 1.1*fyk/gammaM*(b_r-n*d)*d_pl*d_pl/4;
		final double NRd = getSchraubenfestigkeit(d_schr);
		final double c1 = e4-ue-t_r-(d_sch/4+d_pl/2);
		final double c3 = d_sch/2+d_pl;
/*
        System.out.println("c1 = "+c1);
        System.out.println("c3 = "+c3);
        System.out.println("M(1,pl,d) = "+M1pld);
        System.out.println("M(2,pl,d) = "+M2pld);
        System.out.println("N(R,d) = "+NRd);
*/


		//Versagenszustnde
		double Zt = -1.0;
		boolean versagt = false;
	
		//Versagen 1
		double Zt1 = c3*n*NRd/(c3+c1);
/*
        System.out.println("Versagenszustand 1 Z(t1) = "+Zt1);
        System.out.println("Grenzwert  c1*Zt1 = "+ c1*Zt1);
*/
		if(c1*Zt1 >= 0  && c1*Zt1 <= M2pld){
			Zt = Zt1;
			versagt = true;
		}
	
		//Versagen 2
		if(true){//versagt == false){
			double Zt2 = M2pld/c1;
/*
        System.out.println("Versagenszustand 2 Z(t2) = "+Zt2);
        System.out.println("Grenzwert  n*NRd-M2pld/c3 = "+ (n*NRd-M2pld/c3));
*/
			if(Zt2 <= n*NRd-M2pld/c3 && Zt2 >= Zpld){
				Zt = Zt2;
				versagt = true;
			}
		}
	
		//Versagen 3
		if(true){//versagt == false){
			double Zt3 = Zpld*Zpld/(2*M1pld)*(-c1-c3+
				Math.sqrt((c1+c3)*(c1+c3)+4*M1pld/(Zpld*Zpld)*(M1pld+n*NRd*c3)));
/*
        System.out.println("Versagenszustand 3 Z(t3) = "+Zt3);
        System.out.println("Grenzwert  (n*NRd-Zt3)*c3 = "+ (n*NRd-Zt3)*c3);
*/
			if((n*NRd-Zt3)*c3 >= 0 && (n*NRd-Zt3)*c3 <= M2pld){
				Zt = Zt3;
				versagt = true;
			}
		 }
	
		//Versagen 4
		if(true){//versagt = false){
			double Zt4 = Zpld*Zpld/(2*M1pld)*(-c1+
				Math.sqrt(c1*c1+4*M1pld/(Zpld*Zpld)*(M1pld+M2pld)));
/*
        System.out.println("Versagenszustand 4 Z(t4) = "+Zt4);
        System.out.println("Grenzwert  n*NRd = "+ n*NRd);
*/
			if(Zt4 <= n*NRd)
				Zt = Zt4;
		}

/*
        System.out.println("Versagen Z(t) in [MN] = "+ Zt);
*/

        //Schubversagen der Stirnplatte
        if(Zt > b_r*d_pl*fyk/(1.73205*gammaM))
            Zt = b_r*d_pl*fyk/(1.73205*gammaM);

		//Nachweise
        //Zug
        eck.setTragFlansch(Zrd/Zt);

        //Schub
        eck.setTragQuer(Vr/(n*getScherfestigkeit(d_schr)));


    }

    /**
     * Liefert die Zugfestigkeit von Schrauben
     * der Festigkeitsklasse 10.9 in [MN]
     * @param d - Bezeichnung der Schraube nach DIN 18800
     *  z.B M12 oder M27
     * @return Zugfestigkeit in [MN]
     */
    private double getSchraubenfestigkeit(String d){
        if(d.equals("M12"))
            return 61.3/1000.0;
        if(d.equals("M16"))
            return 114/1000.0;
        if(d.equals("M20"))
            return 178/1000.0;
        if(d.equals("M22"))
            return 220/1000.0;
        if(d.equals("M24"))
            return 257/1000.0;
        if(d.equals("M27"))
            return 334/1000.0;
        if(d.equals("M30"))
            return 408/1000.0;
        if(d.equals("M36"))
            return 594/1000.0;
        return -1.0;
    }

    /**
     * Liefert die Scherfestigkeit von Schrauben
     * der Festigkeitsklasse 10.9 in [MN]
     * @param d - Bezeichnung der Schraube nach DIN 18800
     *  z.B M12 oder M27
     * @return Scherfestigkeit in [MN]
     */
    private double getScherfestigkeit(String d){
        if(d.equals("M12"))
            return 56.5/1000.0;
        if(d.equals("M16"))
            return 101/1000.0;
        if(d.equals("M20"))
            return 157/1000.0;
        if(d.equals("M22"))
            return 190/1000.0;
        if(d.equals("M24"))
            return 226/1000.0;
        if(d.equals("M27"))
            return 287/1000.0;
        if(d.equals("M30"))
            return 354/1000.0;
        if(d.equals("M36"))
            return 509/1000.0;
        return -1.0;
    }


}
