/* Generated by Together */

package bemessung.stahltraeger;

/**
 * Frt einen Spannungsnachweis durch.
 * @author Johannes Hhne
 * geprft und fr gut befunden
 */

class Spannungsnachweis implements NachweisVisitor {

    public void visiteBalkenBean(BalkenBean balken){
		//vorgegebene Werte
        double gamma 	= balken.getGammaM();
		double N     	= Math.abs(balken.getN());
        double M		= Math.abs(balken.getM());
        double V		= Math.abs(balken.getQ());
        double fyk    	= balken.getFyk();
        double h		= balken.getH();
        double t		= balken.getT();
        double r		= balken.getR();
        double s		= balken.getS();
        double vorhBT	= balken.getB_t_vorhanden();
        double Wpl		= balken.getWply();
		double A		= balken.getA();

		//zu errechnende Gren
        double a = (1+(gamma*N)/(fyk*(h-2*t-2*r)*s))/2;
		double grenzBT = 37/a*Math.sqrt(240/(fyk));

        if(grenzBT < vorhBT){
            balken.setSpannungsNachweis(-1.0);
            return;
        }

        //plastische Grenzbelastungen in [MN] oder [MNm]
		double Vpl		= fyk/(gamma*Math.sqrt(3))*(h-t)*s / 1000000;
        double Mpl		= fyk/gamma * Wpl*1000 / 1000000 /1000;
        double Npl		= fyk/gamma * A*100 / 1000000;

        // 1. Zeile 1. Spalte
        if(N/Npl<=0.1 && V/Vpl<=0.33){
			balken.setSpannungsNachweis(M/Mpl);
            return;
        }

        // 1. Zeile 2. Spalte
        if(N/Npl<=0.1 && 0.33<V/Vpl && V/Vpl<=1.0){
            balken.setSpannungsNachweis(0.88*M/Mpl+0.37*V/Vpl);
            return;
        }

        // 2. Zeile 1. Spalte
        if(0.1<N/Npl && N/Npl<=1.0 && V/Vpl<=0.33){
            balken.setSpannungsNachweis(0.9*M/Mpl+N/Npl);
            return;
        }

        // 2. Zeile 2. Spalte
        if(0.1<N/Npl && N/Npl<=1.0 &&  0.33<V/Vpl && V/Vpl<=1.0){
            balken.setSpannungsNachweis(0.8*M/Mpl+0.89*N/Npl+0.33*V/Vpl);
            return;
        }

        // Rest
        if(1.0<=N/Npl || 1.0<=M/Mpl || 1.0<=V/Vpl){
            balken.setSpannungsNachweis(Math.max(N/Npl , Math.max(M/Mpl , V/Vpl)));
            return;
        }

        // Unfall
        balken.setSpannungsNachweis(-1.0);
    }

}
