/* Generated by Together */

package gui;

import javax.swing.JPanel;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.GeneralPath;
import java.awt.geom.AffineTransform;
import java.awt.Dimension;
import java.awt.BasicStroke;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.image.BufferedImage;
import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.Point;

/**
 * Fr die eingabe der Lasten ist dieses Panel die Ilustration
 * @author Johannes Hhne
 */
class JPanelSchnittRahmen extends JPanel {
	/**
     * Konstruktor mit allen ntigen Werten fr einen Grundrahmen
     * @param keller - Hhe des Kellers in [m]
     *        links  - Hhe des linken Stiels in [m]
     *        oben   - Breite des Riegels in [m]
     *        rechts - Hhe des rechten Stiels in [m]
     *        gelLinks - Gelenk oben links [boolean]
     *        gelRechts - Gelenk oben rechts [boolean]
     */
    JPanelSchnittRahmen(double keller ,
                       double links ,
                       double oben ,
                       double rechts ,
                       boolean gelLinks ,
                       boolean gelRechts){
        initGUI();
        setH_keller(keller);
        setH_stielLinks(links);
        setB_riegel(oben);
        setH_stielRechts(rechts);
        setGelLinks(gelLinks);
        setGelRechts(gelRechts);
    }


    JPanelSchnittRahmen(){
        initGUI();
    }

    /** This method is called from within the constructor to
     * initialize the form. */
    private void initGUI() {
        this.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        this.setPreferredSize(new Dimension(400,300));
		this.repaint();
    }

    /**
     * ber vererbungsinverse Ablikation knnen hier die Schnittkrfte
     * gezeichnet werden.
     * @param fixPktx - Fixpunkt unten links x - koordinate
     *        fixPkty - Fixpunkt unten links y - koordinate
     *        scale   - Scalierungsfaktor von [m] auf Pix
     *        g2      - Graphics Referenz
     */
    protected void setSchnittgroessen(float fixPktx ,
                                      float fixPkty ,
                                      float scale ,
                                      Graphics2D g2){}

    /**
     * paint berschreiben
     */
    public void paint(Graphics g){
		//Allgemeinen Parameter
        super.paint(g);
		Graphics2D g1 = (Graphics2D)g;
		scale();
        l = this.getWidth();
		h = this.getHeight();


		BufferedImage image = new BufferedImage(l,h,BufferedImage.TYPE_INT_RGB);
        Graphics2D g2 = image.createGraphics();
        g2.setBackground(getBackground());
        g2.setColor(getForeground());
		g2.clearRect(0,0,l,h);
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING , RenderingHints.VALUE_ANTIALIAS_ON);
		//Umrandung
        g2.draw3DRect(0,0,l,h,true);


		//Rahmen selbst
        GeneralPath p = new GeneralPath();
        p.moveTo((float)fixPktx ,
                 (float)fixPkty);
        p.lineTo((float)fixPktx ,
                 (float)fixPkty - (getH_keller()+getH_stielLinks())*scale);
        p.lineTo((float)fixPktx + getB_riegel()*scale ,
                 (float)fixPkty - (getH_keller() + getH_stielRechts())*scale);
        p.lineTo((float)fixPktx + getB_riegel()*scale ,
                  (float)fixPkty -getH_keller()*scale );

        g2.setStroke(new BasicStroke((int)(scale*0.2)));
        g2.draw(p);


        //Lager
        setLagerMom((int)(fixPktx),(int)(fixPkty), g2);
        setLagerMom((float)(fixPktx + getB_riegel()*scale) ,
                    (float)(fixPkty -getH_keller()*scale) , g2);

        setLagerGleit((float)(fixPktx) , (float)(fixPkty - getH_keller()*scale), g2);

        //Momentengelenke
        if(gelLinks)
            setMomGel((float)fixPktx ,
                        (float)(fixPkty-(getH_keller()+getH_stielLinks())*scale), g2);
        if(gelRechts)
            setMomGel((float)(fixPktx + getB_riegel()*scale) ,
                        (float)(fixPkty-(getH_keller()+getH_stielRechts())*scale), g2);

        //ber Vererbungsinverser Aplikation die
        //Schnittkrfte bestimmen
        setSchnittgroessen(fixPktx , fixPkty , scale , g2);
        g2.setColor(Color.black);

        //Bezugslinien
        lin = ((BasicStroke)g2.getStroke()).getLineWidth() > 5 ?
                    2*((BasicStroke)g2.getStroke()).getLineWidth() : 5;
        GeneralPath be = new GeneralPath();
        be.moveTo((float)fixPktx+lin ,
                 (float)fixPkty);
        be.lineTo((float)fixPktx+lin ,
                 (float)fixPkty - (getH_keller()+getH_stielLinks())*scale + lin);
        be.lineTo((float)fixPktx + getB_riegel()*scale - lin ,
                 (float)fixPkty - (getH_keller() + getH_stielRechts())*scale + lin);
        be.lineTo((float)fixPktx + getB_riegel()*scale - lin ,
                  (float)fixPkty -getH_keller()*scale );

        float[] dash = { 2*lin , lin };
        g2.setStroke(new BasicStroke(lin/4 , BasicStroke.CAP_BUTT ,
                     BasicStroke.JOIN_MITER , 1 , dash , 0));
        g2.draw(be);

        g1.drawImage(image,0,0,null);

    }

    /**
     * Hier wird der Rahmen Skaliert, um ihn in die gegebene Abmessung des
     * Rahmens proportiondgetreu einzufgen
     */
    private void scale(){
		double l = this.getWidth()*0.6;
        double h = this.getHeight()*0.6;

        double scale_l = l/this.getB_riegel();
        double scale_h = h/(this.getH_keller() +
                       Math.max(this.getH_stielLinks() , this.getH_stielRechts()));

        scale = (float)Math.min(scale_l , scale_h);
        fixPktx = (float)(0.2*getWidth());
        fixPkty = (float)(0.95*getHeight());
    }

    /**
     * Zeichnet die Momentenlaget
     */
	private void setLagerMom(float xo, float yo, Graphics2D g2){
		int r = (int)(getB_riegel()*scale*0.04);
		int x = (int)xo-r/2;
        int y = (int)yo;
        g2.drawArc(x,y,r,r,0,360);
        g2.drawLine(x  , (int)(yo+r/2)  ,  x-r  , (int)(yo+r/2));
        g2.drawLine(x+r  ,  (int)(yo+r/2)  ,  x+2*r  ,  (int)(yo+r/2));
	}

    /**
     * Zeichnet die Gleitlager
     */
	private void setLagerGleit(float xo, float yo, Graphics2D g2){
		float r = (float)(getB_riegel()*scale*0.02);
        float lin =((BasicStroke)g2.getStroke()).getLineWidth();
        float x = xo+lin;
        float y = yo;
        GeneralPath dr = new GeneralPath();
		dr.moveTo(x,y);
        dr.lineTo(x+2*r,y+r);
        dr.lineTo(x+2*r,y-r);
        dr.closePath();
        dr.moveTo(x+2*r+2*lin,y+r);
        dr.lineTo(x+2*r+2*lin,y-r);
        g2.draw(dr);
	}

    /**
     * Momentengelenke in den Ecken
     */
    private void setMomGel(float xo, float yo, Graphics2D g2){
		int r = (int)(getB_riegel()*scale*0.04);
		int x = (int)xo-r/2;
        int y = (int)yo-r/2;
        Color linie = g2.getColor();
		g2.setColor(g2.getBackground());
        g2.fillArc(x,y,r,r,0,360);
        g2.setColor(linie);
        g2.drawArc(x,y,r,r,0,360);
    }


    /**
     * Hhe des Kellers in [m]
     */
    private double h_keller = 0.0;
    /**
     * Setzt die Hhe des Kellers in [m]
     * @param h_keller - Hhe des Kellers in [m]
     */
    void setH_keller(double h_keller){
        this.h_keller = h_keller;
    	repaint();
    }
    /**
     * Liefert die Hhe des Kellers in [m]
     * @return Hhe des Kellers in [m]
     */
    protected float getH_keller(){return (float)h_keller;}

    /**
     * Hhe des linken Stiels in [m]
     */
    private double h_stielLinks = 0.0;
    /**
     * Setzt die Hhe des linken Stiels in [m]
     * @param h_stielLinks - Hhe des linken Stiels in [m]
     */
    void setH_stielLinks(double h_stielLinks){
        this.h_stielLinks = h_stielLinks;
        repaint();
    }
    /**
     * Liefert die Hhe des linken Stiels in [m]
     * @return Hhe des linken Stiels in [m]
     */
    protected float getH_stielLinks(){return (float)h_stielLinks;}

    /**
     * Hhe des rechten Stiels in [m]
     */
    private double h_stielRechts = 0.0;
    /**
     * Setzt die Hhe des rechten Stiels in [m]
     * @param h_stielRechts - Hhe des rechten Stiels in [m]
     */
    void setH_stielRechts(double h_stielRechts){
        this.h_stielRechts = h_stielRechts;
        repaint();
    }
    /**
     * Liefert die Hhe des rechten Stiels in [m]
     * @return Hhe des rechten Stiels in [m]
     */
    protected float getH_stielRechts(){return (float)h_stielRechts;}

    /**
     * Breite des Riegels in [m]
     */
    private double b_riegel = 0.0;
    /**
     * Setzt die Breite des Riegels in [m]
     * @param b_riegel - Breite des Riegels in [m]
     */
    void setB_riegel(double b_riegel){
        this.b_riegel = b_riegel;
        repaint();}
    /**
     * Liefert die Breite des Riegels in [m]
     * @return Breite des Riegels in [m]
     */
    protected float getB_riegel(){return (float)b_riegel;}

    /**
     * Momentengelenk links oben
     */
    private boolean gelLinks = true;
    /**
     * Setzt Momentengelenk links oben
     * @param gelLinks - Momentengelenk links oben
     */
    protected void setGelLinks(boolean gelLinks){
        this.gelLinks = gelLinks;
        repaint();
    }
    /**
     * Liefert Momentengelenk links oben
     * @return Momentengelenk links oben
     */
    protected boolean getGelLinks(){return gelLinks;}

    /**
     * Momentengelenk rechts oben
     */
    private boolean gelRechts = true;
    /**
     * Setzt Momentengelenk rechts oben
     * @param gelRechts - Momentengelenk rechts oben
     */
    void setGelRechts(boolean gelRechts){
        this.gelRechts = gelRechts;
        repaint();
    }
    /**
     * Liefert Momentengelenk rechts oben
     * @return Momentengelenk rechts oben
     */
    protected boolean getGelRechts(){return gelRechts;}

    /**
     * Liefert die Liniendicke
     * @return Liniendicke des Rahmens
     */
    protected float getLin(){return lin;}


    /**
     * Der Skalierungsfaktor
     */
    private float scale = 1;
    /**
     * Fixpunkt unten rechts
     */
    private float fixPktx = 0;
    private float fixPkty = 0;
    /**
     * Liniendicken
     */
    private float lin = 0;
    /**
     * Breite und Hhe des Panels
     */
    private int l = 0;
	private	int h = 0;

}
