package rahmen;
import java.util.*;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import rahmen.Bauteile;

/**
 * In Objekten der Klasse Rahmen sind alle fr die Bemessung ntigen Daten
 * eines Rahmens gesammelt. Im API werden Instanzen dieser Klasse fr das
 * Abgreifen von Informationen ber Geometrieeigenschaften und Einwirkungs
 * respektive Reaktionskrfte genutzt.
 * @author Johannes Hhne
 */
public abstract class Rahmen implements Serializable  {

	/**
	 * Konstruktor der Klasse
	 */
	Rahmen(double h_links, double h_rechts, double breite, double h_keller, double abstand) {
        linkerStiel.setLaenge(h_links);
        rechterStiel.setLaenge(h_rechts);
        riegel.setLaenge(Math.sqrt(breite*breite + (h_links-h_rechts)*(h_links-h_rechts)));
        riegel.setNeigung(Math.atan((h_rechts-h_links)/breite));
        keller.setLaenge(h_keller);
        this.abstand = abstand;
        this.breite = breite;
    }

    private void writeObject(ObjectOutputStream oos) throws IOException {
        // Write the non-static and non-transient fields of the current class to this stream
        oos.defaultWriteObject();
    }

    private void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException {
        // Read the non-static and non-transient fields of the current class from this stream
        ois.defaultReadObject();
    }


	/**
     * Keller
     * @link aggregationByValue
     */
   	private Keller keller = new Keller();
   	public Keller getKeller(){return keller;}

    /**
     * linker Stiel
     * @link aggregationByValue
     */
    private LinkerStiel linkerStiel = new LinkerStiel();
    public LinkerStiel getLinkerStiel(){return linkerStiel;}

	/**
     * Rechter Stiel
     * @link aggregationByValue
     */
    private RechterStiel rechterStiel = new RechterStiel();
    public RechterStiel getRechterStiel(){return rechterStiel;}

	/**
     * Riegel
     * @link aggregationByValue
     */
    private Riegel riegel = new Riegel();
    public Riegel getRiegel(){return riegel;}

	/**
     * Eigengewicht
     * @link aggregationByValue
     */
    private Eigengewicht eigengewicht = new Eigengewicht();
	Eigengewicht getEigengewicht(){return eigengewicht;}

    /**
     * Scneelast
     * @link aggregationByValue
     */
    private Schnee schnee = new Schnee();
    Schnee getSchneeLast(){return schnee;}

    /**
     * Erddruck
     * @link aggregationByValue
     */
    private Erddruck erddruck = new Erddruck();
    Erddruck getErddruck(){return erddruck;}

    /**
     * Abstand der einzelnen Rahmen in [m]
     */
    private double abstand;
    double getAbstand(){return abstand;}
    void setAbstand(double abstand){this.abstand = abstand;}

    /**
     * Breite des Rahmens in [m]
     */
    private double breite;
    double getBreite(){return breite;}
    void setBreite(double breite){this.breite = breite;}

    /**
     * Die Verdrehung Y1 in den einzelnen Lastkombinationen
     */
    private double[] Y1 = new double[6];
    /**
     * Setzt die Verdrehung Y1 in den einzelnen Lastkombinationen
     * @param index - Lastkombination 0 - 5
     * wert - Wert der Vervormung
     */
    public double[] getY1(){return Y1;}
    /**
     * Lifert die Verdrehung Y1 in den einzelnen Lastkombinationen
     * @return wert der Verformung
     */
    void setY1(int index, double wert){Y1[index] = wert;}

    /**
     * Die Verdrehung Y2 in den einzelnen Lastkombinationen
     */
    private double[] Y2 = new double[6];
    /**
     * Setzt die Verdrehung Y2 in den einzelnen Lastkombinationen
     * @param index - Lastkombination 0 - 5
     * wert - Wert der Vervormung
     */
    public double[] getY2(){return Y2;}
    /**
     * Lifert die Verdrehung Y2 in den einzelnen Lastkombinationen
     * @return wert der Verformung
     */
    void setY2(int index, double wert){Y2[index] = wert;}

    /**
     * Die Verdrehung Y3 in den einzelnen Lastkombinationen
     */
    private double[] Y3 = new double[6];
    /**
     * Setzt die Verdrehung Y3 in den einzelnen Lastkombinationen
     * @param index - Lastkombination 0 - 5
     * wert - Wert der Vervormung
     */
    public double[] getY3(){return Y3;}
    /**
     * Lifert die Verdrehung Y3 in den einzelnen Lastkombinationen
     * @return wert der Verformung
     */
    void setY3(int index, double wert){Y3[index] = wert;}

    /**
     * Die Verdrehung Y4 in den einzelnen Lastkombinationen
     */
    private double[] Y4 = new double[6];
    /**
     * Setzt die Verdrehung Y4 in den einzelnen Lastkombinationen
     * @param index - Lastkombination 0 - 5
     * wert - Wert der Vervormung
     */
    public double[] getY4(){return Y4;}
    /**
     * Lifert die Verdrehung Y4 in den einzelnen Lastkombinationen
     * @return wert der Verformung
     */
    void setY4(int index, double wert){Y4[index] = wert;}


    /**
     * Windlast
     * @link aggregationByValue
     */
    private Wind windlast = new Wind();

    /**
     * @link
     * @shapeType PatternLink
     * @pattern Visitor
     * @supplierRole Visitor
     * @hidden 
     */
    /*# private Schnittgroessen _schnittgroessen; */

    /**
     * @link
     * @shapeType PatternLink
     * @pattern Visitor
     * @supplierRole Concrete elements 
     */
    /*# private Rahmen_I _rahmen_I; */

    /**
     * @link
     * @shapeType PatternLink
     * @pattern Visitor
     * @supplierRole Concrete elements 
     */
    /*# private Rahmen_II _rahmen_II; */

    /**
     * @link
     * @shapeType PatternLink
     * @pattern Visitor
     * @supplierRole Concrete elements 
     */
    /*# private Rahmen_III _rahmen_III; */

    /**
     * @link
     * @shapeType PatternLink
     * @pattern Visitor
     * @supplierRole Concrete elements 
     */
    /*# private Rahmen_IV _rahmen_IV; */

    public Wind getWindlast(){return windlast;}

    abstract void accept(Schnittgroessen visitor);

    abstract void schneiden();
}
