package rahmen;
import java.util.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * Diese Factory stellt die Kommunikation mit der Umwelt her. Sie Sorgt dafr,
 * dass es im System immer nur einen Rahmen gibt, und das dieser wohl
 * deffiniert ist. Die Herstellung eines Rahmens, der alle zugesicherten
 * Vertragsbedingungen einhlt ist kompliziert und sollte deshalb nur ber
 * diese Factory geschehen.
 * Weiterhin sollte man unbedingt die Dokumentation der implementierten
 * Algorythmen kennen, um mit den Bezeichnungen etwas anfangen zu knnen.
 * @author Johannes Hhne
 */

public final class RahmenFactory implements ActionListener {

    /**
     * @label Erzeugt
     * @labelDirection forward
     * @link aggregationByValue 
     */
    protected static Rahmen rahm = null;
    protected static RahmenFactory factory = null;

	/**
     * Dieser Schalter entscheidet darber, ob der fertige Rahmen geliefert wird
     * oder nicht. Fertig ist ein Rahmen nur, wenn er auch geschnitten ist.
     */
    protected boolean erlauben = false;

    /**
     * Konstruktor der Klasse
     */
    protected RahmenFactory() {
    }

    /**
     * Die statische Klasse sorgt dafr, dass es nur eine
     * Factory im System gibt.
     */
    public static RahmenFactory  getFactory(){
        if (factory == null)
            factory = new RahmenFactory();
        return factory;
    }

    /**
     * diese Methode setzt die ueren Abmessungen des Rahmens. sie sollte
     * als erstes bedient werden.
     * @param typ - Der Typ der Halle "Typ I" , "Typ II" , "Typ III" oder "Typ IV"
     * h_links - Die Hhe des linken Stiels in [m]
     * h_rechts - Hhe des rechten Stiels in [m]
     * breite - Breite des Rahmens in [m]
     * h_keller - Hhe des Kellers in [m]
     * abstand - Abstand der einzelnen Rahmen in [m]
     *
     * @return
     * true wenn die Methode erfolgreich war
     * false wenn sie gescheitert ist
     */
    public boolean setGeometrie(String typ,
        						double h_links,
                                double h_rechts,
                                double breite,
                                double h_keller,
                                double abstand){
        rahm = null;
//System.out.println(typ+" h_links = "+h_links+" h_rechts = "+h_rechts+" breite = "+breite+" h_keller = "+h_keller+" abstand = "+abstand );

        erlauben = false;
	   if(typ.equals("Typ I")){
			rahm = new Rahmen_I(h_links, h_rechts, breite, h_keller, abstand);
			return true;
		}
		if(typ.equals("Typ II")){
			rahm = new Rahmen_II(h_links, h_rechts, breite, h_keller, abstand);
			return true;
		}
		if(typ.equals("Typ III")){
			rahm = new Rahmen_III(h_links, h_rechts, breite, h_keller, abstand);
			return true;
		}
		if(typ.equals("Typ IV")){
			rahm = new Rahmen_IV(h_links, h_rechts, breite, h_keller, abstand);
			return true;
		}else
			return false;
    }


    /**
     * Linker Stiel. Die Geometrie der Halle sollte vorher bekannt sein
     * @param i - Flchentrgheit in der Papierebene in [m^4]
     * e - E-Modul des Bauteils in [N/mm^2] = [MN/m^2]
     * @return true - wenn der Vorgang erfolgreich war
     *         false - wenn nicht (z.B. geometrie ist noch nicht gesetzt)
     */
    public boolean setStielLinks(double i, double e) {
        erlauben = false;
        if(rahm == null)
			return false;
        rahm.getLinkerStiel().setI_y(i);
        rahm.getLinkerStiel().setE(e);
//System.out.println("Linker Stiel E = "+e+"    I ="+i);
        return true;
    }

    /**
     * Riegel Die Geometrie der Halle sollte vorher bekannt sein
     * @param i - Flchentrgheit in der Papierebene in [m^4]
     * e - E-Modul des Bauteils in [N/mm^2] = [MN/m^2]
     * @return true - wenn der Vorgang erfolgreich war
     *         false - wenn nicht (z.B. geometrie ist noch nicht gesetzt)
     */
    public boolean setRiegel(double i, double e) {
        erlauben = false;
        if(rahm == null)
			return false;
        rahm.getRiegel().setI_y(i);
        rahm.getRiegel().setE(e);
//System.out.println("Riegel E = "+e+"    I ="+i);
        return true;    }

    /**
     * Rechter Stiel. Die Geometrie der Halle sollte vorher bekannt sein
     * @param i - Flchentrgheit in der Papierebene in [m^4]
     * e - E-Modul des Bauteils in [N/mm^2] = [MN/m^2]
     * @return true - wenn der Vorgang erfolgreich war
     *         false - wenn nicht (z.B. geometrie ist noch nicht gesetzt)
     */
    public boolean setStielRechts(double i, double e) {
        erlauben = false;
        if(rahm == null)
			return false;
        rahm.getRechterStiel().setI_y(i);
        rahm.getRechterStiel().setE(e);
//System.out.println("Rechter Stiel E = "+e+"    I ="+i);
        return true;
    }

    /**
     * Keller. Die Geometrie der Halle sollte vorher bekannt sein
     * @param i - Flchentrgheit in der Papierebene in [m^4]
     * e - E-Modul des Bauteils in [N/mm^2] = [MN/m^2]
     * @return true - wenn der Vorgang erfolgreich war
     *         false - wenn nicht (z.B. geometrie ist noch nicht gesetzt)
     */
    public boolean setKeller(double i, double e) {
        erlauben = false;
        if(rahm == null)
			return false;
        rahm.getKeller().setI_y(i);
        rahm.getKeller().setE(e);
//System.out.println("Keller E = "+e+"    I ="+i);
        return true;
    }

    /**
     * Methode zum setzten der Gewichtskraft aus Eigengewicht und
     * Aufbauten
     * @param links - Gewichtskraft im linken Stiel in [MN/m]
     * riegel - Gewichtskraft im Riegel in [MN/m]
     * rechts - Gewichtskraft im rechten Stiel in [MN/m]
     * @return true - wenn alles erfolgreich war
     *         false - wenn etwas schief gelaufen ist
     * (z. B.: Hallengeometzrie ist noch nicht gesetzt)
     */
    public boolean setEigengewicht(double links, double riegel, double rechts) {
        erlauben = false;
        if(rahm == null)
			return false;
        rahm.getEigengewicht().setG_Links(links);
        rahm.getEigengewicht().setG_Rechts(rechts);
        rahm.getEigengewicht().setG_Riegel(riegel);
//System.out.println("g links = "+links+"   g riegel = "+riegel+"   g rechts ="+rechts);
        return true;
    }

    /**
     * Methode zum Setzen der ntigen Parameter, die fr die Schneelast
     * ntig sind.
     * @param zone - Schneezone. Erlaubt sind folgende Werte "I", "II" ,"III" und "IV"
     * gelHoehe - Hhe des Gelndes in [m]
     * @return true - alles war erfolgreich
     *         false - es ist ein Fehler aufgetreten
     */
    public boolean setSchneeLast(String zone , int gelHoehe){
        erlauben = false;
        if(rahm == null || (!zone.equals("I") && !zone.equals("II") && !zone.equals("III") && !zone.equals("IV")))
			return false;
        rahm.getSchneeLast().setZone(zone);
        rahm.getSchneeLast().setGelHoehe(gelHoehe);
        return true;
    }


    /**
     * Metode zum setzten aller ntiger Parameter fr den Erddruck
     * @param phi - Reibungswinkel des Bodens in []
     * gamma - Eingenlast des Bodens in [MN/m^3]
     * p - Personenauflast in [N/mm^2]
     * @return true - alles war erfolgreich
     *         false - es ist ein Fehler aufgetreten
     */
    public boolean setErddruck(int phi, double gamma, double p){
        erlauben = false;
        if(rahm == null)
			return false;
        rahm.getErddruck().setPhi(phi);
        rahm.getErddruck().setGamma(gamma);
        rahm.getErddruck().setP(p);
//System.out.println("phi ="+phi+"  gamma = "+gamma+"  p = "+p);
        return true;
    }

    /**
     * Wenn ein ActionEvent in das Objekt gefeuert wird, werden die
     * Schnittkrfte bestimmt. Danach wird der fertige Rahmen freigegeben.
     * Dies ist keine orthodoxe Vorgehensweise, erscheint mir aber als
     * durchaus sicher, da erst der geschnittene Rahmen in den Augen des
     * Ingenieurs wirklich fertig ist.
     * @param ev - ActionEvent
     */
    public void  actionPerformed(ActionEvent ev){
        if(rahm != null){
    		rahm.schneiden();
        	erlauben = true;}
    }

    public Rahmen getRahmen(){
        if(erlauben == true)
            return rahm;
        return null;
    }
}
